# Example Mandelbrot-escape program
#

from graphics import *


# mshow is the function that visualizes whether
# a point escapes or not...
def mshow():
    """ mshow is a program that visualizes whether or
        not a point, c (the first click) escapes
        under the update rule z <- z**2 + c
    """
    # instructions!
    print( "Click outside the 'infinity circle' to quit.")
    
    # create the window
    win = GraphWin("Mandelbrot Demo",600,400)
    win.setCoords(-2,-1,1,1)
    
    # load the M. Set image
    i = Image( Point(-0.5,0), "test.gif" )
    i.draw(win)
    
    # draw the axes
    imag_axis = Line( Point(0,-1), Point(0,1) )
    imag_axis.setOutline( "yellow" )
    imag_axis.draw(win)
    real_axis = Line( Point(-2,0), Point(1,0) )
    real_axis.setOutline( "yellow" )
    real_axis.draw(win)

    # the circle at the origin with radius 2 ("infinity circle"!)
    infinity_circle = Circle( Point( 0,0), 2.0 )
    infinity_circle.setOutline( "orange" )
    infinity_circle.draw(win)

    # prompt user to choose c by clicking
    print("Click to choose c.")
    p = win.getMouse()
    print( "c is", p.x, "+", p.y, "* 1j\n")
    c = p.x + p.y*1j

    # keep track of this c as orig_c
    orig_c = p.x + p.y*1j
    orig_cir = Circle( p, 0.025 )
    orig_cir.setFill( "cyan" )
    orig_cir.draw( win )

    # here is z_0 (the initial value of z)
    z = 0 + 0*1j

    # count our # of iterations
    cur_iter = 0
    while True:
        # let the user know what z is... and plot in yellow
        print( "iter #", cur_iter, ": z =", round(z.real,4), "+", round(z.imag,4), "* 1j")
        cir = Circle( Point( z.real, z.imag ), 0.025 )
        cir.setFill( "yellow" )
        cir.draw( win )

        # click for the next iteration
        click_p = win.getMouse()

        # leave if we click outside the infinity circle
        x = click_p.x
        y = click_p.y
        if ( x**2 + y**2 )**0.5 > 2.0: break

        # special case so that c always ends up as cyan
        if cur_iter == 1:
            cir.setFill( "cyan" )
        else:
            cir.setFill( "red" )
            
        # update with the Mandelbrot rule!
        z = z**2 + c
        cur_iter += 1

    print( "Click once more to close...")
    win.getMouse()
    win.close()


mshow()